import pickle
from statsmodels.tsa.ar_model import AutoReg
from sklearn.metrics import mean_squared_error
from math import sqrt
from matplotlib import pyplot as plt
from Open_Files import create_directory


def autoregressor(train, test, lag, title, plotting=False):
    # We create an AR model and fit it with the train data. Once it is fitted, we extract the AR parameters
    window = lag
    model = AutoReg(train, lags=lag)
    model_fit = model.fit()
    coef = model_fit.params
    history = train[len(train) - window:]
    history = [history[i] for i in range(len(history))]
    predictions = list()
    # We predict each value of the test data manually
    for t in range(len(test)):
        length = len(history)
        lag = [history[i] for i in range(length - window, length)]
        yhat = coef[0]
        for d in range(window):
            yhat += coef[d + 1] * lag[window - d - 1]
        obs = test[t]
        yhat = yhat
        predictions.append(yhat)
        history.append(obs)
    # We calculate the RMSE value of our prediction
    rmse = sqrt(mean_squared_error(test, predictions))
    if plotting:
        plt.plot(test)
        plt.title(f"{title} 10000 value Predictions for lag {window}")
        plt.plot(predictions, color='red')
        plt.ylim([-10, 10])
        create_directory(title)
        plt.savefig(f"{title}/{title}_10000_Prediction_lag_{window}.png")
        plt.clf()
        plt.plot(test[:400])
        plt.title(f"{title} 400 value Predictions for lag {window}")
        plt.plot(predictions[:400], color='red')
        plt.ylim([-10, 10])
        plt.savefig(f"{title}/{title}_400_Prediction_lag_{window}.png")
        plt.clf()
    return rmse, coef


def run_autoregressor(data, title, name, accepted_rmse=0.001, plotting=False):
    coefs = []
    # For each channel in the data, we split it into train-test, with test only having 2000 datapoints
    for i in range(len(data)):
        train, test = data[i][1:len(data[i]) - 2000], data[i][len(data[i]) - 2000:]
        rmses = []
        rmse = 1
        lag = 0
        # We apply the AR model with lag 1 on the channel. This is repeated, while increasing the lag, until either the
        # RMSE of the AR prediction is lower than the set RMSE threshold or lag is over 25.
        while rmse > accepted_rmse and lag < 25:
            rmse, coef = autoregressor(train, test, lag, title, plotting)
            rmses.append(rmse)
            lag = lag + 1
        # We plot the RMSE values for each lag
        plt.plot(rmses)
        plt.ylim([0, 1])
        plt.title(f"{name} RMSE values for channel {i}")
        create_directory(title)
        plt.savefig(f"{title}/{name}_RMSE_values_channel_{i}.png")
        plt.clf()
        coefs.append(coef)
    # We save the AR matrix
    create_directory("Autoreg_Matrixes")
    f = open(f'Autoreg_Matrixes/{name}_autoreg_matrix.pckl', 'wb')
    pickle.dump(coefs, f)
    f.close()
