from matplotlib import pyplot as plt
from pandas.plotting import lag_plot
import numpy as np
import pandas as pd
from Open_Files import create_directory


def autocorrelate_fft(data):
    # Autocorrelation based on the real part of the Fourier's Transform
    size = 2 ** np.ceil(np.log2(2*len(data) - 1)).astype('int')
    var = np.var(data)
    ndata = data - np.mean(data)
    fft = np.fft.fft(ndata, size)
    pwr = np.abs(fft) ** 2
    acorr = np.fft.ifft(pwr).real / var / len(data)
    return acorr[:len(data)]


def autocorrelate_numpy(data):
    # Autocorrelation based on the naive correlatio method
    mean = np.mean(data)
    var = np.var(data)
    ndata = data - mean
    acorr = np.correlate(ndata, ndata, 'full')[len(ndata)-1:]
    acorr = acorr / var / len(ndata)
    return acorr


def autocorrelate(data):
    # We calculate the correlations between channels
    df = pd.DataFrame(data.T)
    corr_values = df.corr()
    return corr_values


def autocorrelate_matrix(corr_val, title):
    # We show the channel correlation matrix
    plt.matshow(corr_val)
    plt.colorbar()
    plt.title(title+" autocorrelation matrix")
    create_directory(f"{title}/{title}_Autocorrelate")
    plt.savefig(f"{title}/{title}_Autocorrelate/Autocorrelation_matrix_{title}.png")
    plt.clf()


def autocorrelate_plot_signal(data, signal, title):
    # We get the autocorrelation coefficients using the real Fourier's Transform
    autocorr_signal = autocorrelate_fft(data)
    # We plot the first 1000 autocorrelation values
    plt.stem(autocorr_signal[:1000])
    plt.title(f"{title} Signal {signal}")
    create_directory(f"{title}/Autocorrelate_{title}")
    plt.savefig(f"{title}/Autocorrelate_{title}/{title}_{signal}_1000.png")
    plt.clf()
    # We plot the first 100 autocorrelation values
    plt.stem(autocorr_signal[:100])
    plt.title(f"{title} Signal {signal}")
    plt.savefig(f"{title}/Autocorrelate_{title}/{title}_{signal}_100.png")
    plt.clf()
    for lag in [1, 20]:
        # We plot the lag autocorrelation for lag 1 and lag 20
        data_series = pd.Series(data)
        lag_plot(data_series, lag=lag)
        plt.title(f'{title} Lag Plot for lag {lag} channel {signal}')
        create_directory(f"{title}/{title}_lag_plot")
        plt.savefig(f"{title}/{title}_lag_plot/{title}_lag_plot_channel_{signal}_lag{lag}.png")
        plt.clf()


def autocorrelate_plot_block(data, title):
    # We plot the autocorrelation of every channel of the dataset
    for signal in range(len(data)):
        autocorrelate_plot_signal(data[signal], signal, title)

