from matplotlib import pyplot as plt
import numpy as np
from sklearn.decomposition import PCA
from Open_Files import create_directory


def run_PCA(data, accepted_variance, title):
    # We use the PCA implementation from sklearn.decomposition
    pca = PCA()
    data = np.asarray(data)
    # We transform the dataset using the PCA technique, and return the ordered explained variance of each channel
    pca.fit_transform(data.T)
    pca_explained_values = pca.explained_variance_ratio_
    # We plot the explained value of each channel
    plt.plot(pca_explained_values)
    plt.title(f"{title} PCA ratio")
    # We check if the directory were we are saving the figure exists. If it does not, we create it
    create_directory(title)
    plt.savefig(f"{title}/{title}_PCA_ratio.png")
    plt.clf()
    pca_added_values = []
    min_components = -1
    # We choose as many channels as necessary to surpass the accepted explained variance
    for i in range(len(pca_explained_values)):
        pca_added_values.append(np.sum(pca_explained_values[:i]))
        if pca_added_values[i] >= accepted_variance and min_components == -1:
            min_components = i
    # We plot the added explained value
    plt.plot(pca_added_values)
    plt.title(f"{title} PCA ratio added")
    plt.savefig(f"{title}/{title}_PCA_ratio_added.png")
    plt.clf()
    pca = PCA(min_components)
    principalComponents = pca.fit_transform(data.T)
    return principalComponents.T


def plot_PCA(PCA, title):
    # We plot all the PCA channels
    plt.figure(figsize=(20, 40))
    plt.subplot(3, 1, 2)
    plt.plot([[np.nan for i in range(len(PCA))]])
    plt.plot(PCA.T[:2000] / 20 + [0.5 * i for i in range(1, len(PCA) + 1)])
    plt.yticks([0.5 * i for i in range(1, len(PCA) + 1)], [1 * i for i in range(len(PCA))])
    plt.ylabel('PCA data')
    create_directory(title)
    plt.savefig(f"{title}/{title}_PCA.png")
    plt.clf()
