from statistics import mean
import scipy.signal as spsg
from matplotlib import pyplot as plt
import numpy as np
from Open_Files import create_directory


def plot_ICA(ICA, title):
    # We plot the first 40000 ICA values of all the channels
    plt.figure(figsize=(20, 40))
    plt.subplot(3, 1, 2)
    plt.plot([[np.nan for i in range(len(ICA))]])
    plt.plot(ICA.T[:40000] / 20 + [0.5 * i for i in range(1, len(ICA) + 1)])
    plt.yticks([0.5 * i for i in range(1, len(ICA) + 1)], [1 * i for i in range(len(ICA))])
    plt.ylabel('ICA data')
    plt.title(title)
    # We check if the directory were we are saving the figure exists. If it does not, we create it
    create_directory(title)
    plt.savefig(f"{title}/{title}_ICA_plotted.png")
    plt.clf()


def subsample_signal(data, factor):
    # We apply subsampling by a factor to a channel
    sub_data = [None] * int(len(data) / factor)
    for i in range(int(len(data) / factor)):
        sub_data[i] = mean(data[i * factor:i * factor + factor])
    return sub_data


def subsample_block(data, factor):
    # We apply subsampling by a factor to each channel
    sub_data = [None] * len(data)
    for i in range(len(data)):
        sub_data[i] = subsample_signal(data[i], factor)
    return sub_data


def get_separated_freqs(freqs, freq_band):
    n_order = 3
    sampling_freq = 1000.
    nyquist_freq = sampling_freq / 2.
    # 0 indicates that we chose the unfiltered dataset
    if freq_band == 0:
        return np.array(freqs)
    # 1 indicates that we chose the alpha dataset
    if freq_band == 1:
        low_f = 8. / nyquist_freq
        high_f = 12. / nyquist_freq
    # 2 indicates that we chose the beta dataset
    elif freq_band == 2:
        low_f = 12. / nyquist_freq
        high_f = 30. / nyquist_freq
    # 3 indicates that we chose the gamma dataset
    elif freq_band == 3:
        low_f = 30. / nyquist_freq
        high_f = 80. / nyquist_freq
    else:
        raise NameError('unknown filter')
    # We create the bandpass with the right parameters, as indicated by the frequency band chosen
    b, a = spsg.iirfilter(n_order, [low_f, high_f], btype='bandpass', ftype='butter')
    filtered_ts = []
    # We apply the filter to each channel
    for freq in freqs:
        filtered_ts.append(spsg.filtfilt(b, a, freq, axis=0))
    return np.array(filtered_ts)

