import pickle
import json
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
from statistics import mean
from os.path import exists

import Signal_Help
import PCA
import Open_Files
import Autocorrelate
import AR_Model


def main(accepted_variance, freqs, accepted_rmse=None, plotting=False):
    # We use 0.001 as a default RMSE value, as it displays correctly the covariation of minimum necessary lag between
    # brainwave signal
    if accepted_rmse is None:
        accepted_rmse = [0.001]
    overall_alpha, overall_beta, overall_gamma = [], [], []
    for acc_rmse in accepted_rmse:
        print(acc_rmse)
        # We get the minimum necessary lag for each signature for unu, bidi_cons and bidi_conf
        alpha_uni, beta_uni, gamma_uni = P2uni(accepted_variance, freqs, acc_rmse)
        print("Uni")
        alpha_bidi, beta_bidi, gamma_bidi = P2bidi(accepted_variance, freqs, acc_rmse)
        print("Bidi")
        alpha, beta, gamma = alpha_uni + alpha_bidi, beta_uni + beta_bidi, gamma_uni + gamma_bidi
        # We show the mean minimum necessary lag and error for each signature throughout every experimental condition
        plot_errors([alpha], [beta], [gamma], [acc_rmse], "Overall")
        # We show the visual representation of the past data each siganture considers
        plot_lag_selection_comparison(int(np.mean(alpha)), int(np.mean(beta)), int(np.mean(gamma)), acc_rmse)
        overall_alpha.append(alpha)
        overall_beta.append(beta)
        overall_gamma.append(gamma)
    # We show the mean minimum necessary lag for alpha, beta and gamma and error throughout every RMSE value
    plot_errors(overall_alpha, overall_beta, overall_gamma, accepted_rmse, "Compounded")


def P2uni(accepted_variance, freqs, accepted_rmse, plotting=False):
    # We get the preprocessed uni data separated by metro
    p2uniICA, p2uniICA_A, p2uniICA_B = Open_Files.open_ICA_uni()
    metros = {"P2uniICA_A": p2uniICA_A, 'P2uniICA_B': p2uniICA_B}
    # We apply the process and AR and get the mean minimum necessary lag values of each signature for the uni dataset
    alpha, beta, gamma = run_tests(accepted_variance, freqs, accepted_rmse, metros)
    plot_errors([alpha], [beta], [gamma], [accepted_rmse], "Uni")
    return alpha, beta, gamma


def P2bidi(accepted_variance, freqs, accepted_rmse=0.001, plotting=False):
    # We get the preprocessed bidi data separated by metro
    p2bidiICA_cons, p2bidiICA_A_cons, p2bidiICA_B_cons = Open_Files.open_ICA_bidi_cons()
    metros = {"P2bidiICA_A_cons": p2bidiICA_A_cons, 'P2bidiICA_B_cons': p2bidiICA_B_cons}
    # We apply the process and AR and get the mean minimum necessary lag values of each signature for both bidi datasets
    alpha_cons, beta_cons, gamma_cons = run_tests(accepted_variance, freqs, accepted_rmse, metros)
    p2bidiICA_conf, p2bidiICA_A_conf, p2bidiICA_B_conf = Open_Files.open_ICA_bidi_conf()
    metros = {"P2bidiICA_A_conf": p2bidiICA_A_conf, 'P2bidiICA_B_conf': p2bidiICA_B_conf}
    alpha_conf, beta_conf, gamma_conf = run_tests(accepted_variance, freqs, accepted_rmse, metros)
    # We plot the mean minimum necessary lag values
    plot_errors([alpha_cons], [beta_cons], [gamma_cons], [accepted_rmse], "Bidi Cons")
    plot_errors([alpha_conf], [beta_conf], [gamma_conf], [accepted_rmse], "Bidi Conf")
    # We create a bidi database, which contains the bidi_cons and bidi_conf, and plot them
    alpha, beta, gamma = alpha_cons + alpha_conf, beta_cons + beta_conf, gamma_conf + gamma_cons
    plot_errors([alpha], [beta], [gamma], [accepted_rmse], "Bidi")
    return alpha, beta, gamma


def run_tests(accepted_variance, freqs, accepted_rmse, metros, plotting=False):
    base, alpha, beta, gamma = ("", 0), ('_alpha', 1), ('_beta', 2), ('_gamma', 3)
    frequencies = []
    if 0 in freqs:
        frequencies.append(base)
    if 1 in freqs:
        frequencies.append(alpha)
    if 2 in freqs:
        frequencies.append(beta)
    if 3 in freqs:
        frequencies.append(gamma)
    alpha = []
    beta = []
    gamma = []
    unfiltered = []
    for key in metros:
        print(key)
        lag_matrix = {}
        for freq_name, freq in frequencies:
            print(freq)
            new_key = key + freq_name
            # We check if the dataset we are running the AR on already has a PCA dataset. If not, we create one
            if exists(f'{new_key}/{new_key}_PCA_data.pckl'):
                pca_uniICA = pd.read_pickle(fr'{new_key}/{new_key}_PCA_data.pckl')
            else:
                data = Signal_Help.get_separated_freqs(metros[key], freq)
                Signal_Help.plot_ICA(data, new_key)
                pca_uniICA = PCA.run_PCA(data, accepted_variance, new_key)
                f = open(f'{new_key}/{new_key}_PCA_data.pckl', 'wb')
                pickle.dump(pca_uniICA, f)
                f.close()
                PCA.plot_PCA(pca_uniICA, new_key)
            # We calculate the correlation between channels for the dataset and plot the matrix
            p2uni_corr = Autocorrelate.autocorrelate(pca_uniICA)
            Autocorrelate.autocorrelate_matrix(p2uni_corr, new_key)
            Autocorrelate.autocorrelate_plot_block(pca_uniICA, new_key)
            if accepted_rmse != 0.001:
                rmse_new_key = new_key + "_" + str(accepted_rmse)
                rmse_key = key + "_" + str(accepted_rmse)
            else:
                rmse_new_key = new_key
                rmse_key = key
            # If we have not saved the AR matrix for that dataset wit that RMSE value, we run the autoregressive model
            if not exists(f'Autoreg_Matrixes/{rmse_new_key}_autoreg_matrix.pckl'):
                AR_Model.run_autoregressor(pca_uniICA, new_key, rmse_new_key, accepted_rmse, plotting)
            matrix = pd.read_pickle(f'Autoreg_Matrixes/{rmse_new_key}_autoreg_matrix.pckl')
            num_lags = []
            # We save the minimum necessary number of lags
            for row in matrix:
                num_lags.append(len(row))
            if freq == 0:
                unfiltered.append(len(row))
            elif freq == 1:
                alpha.append(len(row))
            elif freq == 2:
                beta.append(len(row))
            elif freq == 3:
                gamma.append(len(row))
            # We save in a dictionary the total mean necessary number of lags for that brainwave signature
            lag_matrix[new_key] = mean(num_lags)
            matrix_json = {}
            for i in range(len(matrix)):
                matrix_json[i] = list(matrix[i])
            out_file = open(f'Autoreg_Matrixes/{rmse_new_key}_autoreg_matrix.json', "w")
            json.dump(matrix_json, out_file)
            out_file.close()
        Open_Files.create_directory('Lag_Matrixes')
        out_file = open(f'Lag_Matrixes/{rmse_key}_lag_matrix.json', "w")
        json.dump(lag_matrix, out_file)
        out_file.close()
    return alpha, beta, gamma


def plot_errors(alpha, beta, gamma, accepted_rmse, title):
    labels = ['Alpha', 'Beta', 'Gamma']
    x_pos = np.arange(len(labels))
    for i in range(len(alpha)):  # We run through all the values
        # We calculate the mean values of the alpha, beta and gamma minimum necessary lags
        alpha_mean = np.mean(alpha[i])
        beta_mean = np.mean(beta[i])
        gamma_mean = np.mean(gamma[i])
        # We calculate the mean values of the alpha, beta and gamma minimum necessary lags
        alpha_std = np.std(alpha[i])
        beta_std = np.std(beta[i])
        gamma_std = np.std(gamma[i])
        # We plot the mean and error of each minimum necessary lag for each signature in a plot bar
        means = [alpha_mean, beta_mean, gamma_mean]
        error = [alpha_std, beta_std, gamma_std]
        plt.bar(x_pos + (0.15 * i), means, yerr=error, align='center', alpha=0.5, ecolor='black', capsize=10,
                label=accepted_rmse[i], width=0.15)
    plt.legend()
    plt.xticks(x_pos + 0.15 * i / 2, labels)
    plt.title(f'Lag values with STD error of {title} for {accepted_rmse}')
    plt.grid(axis="y")
    plt.savefig(f"Plotted_Lag_Values_{title}_{accepted_rmse}.png")
    plt.clf()


def plot_lag_selection_comparison(alpha, beta, gamma, accepted_rmse):
    # We select a channel for each signature
    alpha_channel = pd.read_pickle("P2uniICA_A_alpha/P2uniICA_A_alpha_PCA_data.pckl")[0][:500] / 20
    beta_channel = pd.read_pickle("P2uniICA_A_beta/P2uniICA_A_beta_PCA_data.pckl")[0][:500] / 20
    gamma_channel = pd.read_pickle("P2uniICA_A_gamma/P2uniICA_A_gamma_PCA_data.pckl")[0][:500] / 20
    # We plot a channel of each signature, and mark in red the amount of datapoints each signature retains
    plt.plot(alpha_channel + 0.15, label="Alpha")
    plt.plot(alpha_channel[:alpha] + 0.15, color="red")
    plt.plot(beta_channel, label="Beta")
    plt.plot(beta_channel[:beta], color="red")
    plt.plot(gamma_channel - 0.15, label="Gamma")
    plt.plot(gamma_channel[:gamma] - 0.15, color="red", label="Selected Data")
    plt.title("Visual representation of data points selected for RMSE value of " + str(accepted_rmse))
    plt.legend()
    plt.savefig(f"Lag_selection_comparison_{accepted_rmse}.png")
    plt.clf()
